;************************************************
;* DS1820 subroutines for AVR's			*
;*						*
;* Copyright 2002-2003  Hans OPSTAELE		*
;*   based on code from Wayne Peacock		*
;*						*
;* Displays temperature in .5 C resolution	*
;* on a 1x16LCD panel and controls a plotter 	*
;************************************************
;* See Schematic for Hardware connections	*
;************************************************


; **** Pin Consignments ****

.equ	WIRE		= 4

.set	WIREPORT	= PORTB
.set	WIREDIR		= DDRB
.set	WIREIN		= PINB


.def	templsb		= r2
.def	tempmsb		= r3
.def	userb1		= r4
.def	userb2		= r5
.def	crap1		= r6
.def	crap2		= r7
.def	count_remain	= r8
.def	count_per_c	= r9
.def	scratch_crc	= r10
.def	crc		= r11
;.def	templ		= r12		; Converted result
;.def	temph		= r13

;.def	vallow		= r22
;.def	valhigh		= r23

;************************************************
;* DS1820 Init	(4 Mhz)				*
;************************************************

DS1820_init:

	cbi	WIREPORT, WIRE		; Bit always low
	cbi	WIREDIR, WIRE		; Set pin up as open drain
	ret

	
;************************************************
;* DS1820 Reset	(4 Mhz)				*
;*						*
;* T clear if Presence 				*
;* Requires:	temp				*
;* 		count1 (timer)			*
;************************************************

.equ	T	= 6	; STATUS REG

DS1820_reset:
	
	; Reset Pulse
	sbi	WIREDIR, WIRE		; Bus Low
	ldi	count1, 137			
	rcall	delayX5us		; Wait 720us
	cbi	WIREDIR, WIRE		; Bus High
	
	; Delay
	ldi	count1, 13
	rcall	delayX5us		; Wait 67.5us
	
	; Check for Presence signal
	in	temp, WIREIN 
	bst	temp, WIRE		; Store bus status
	
	; Delay
	ldi	count1, 190
	rcall	delayX5us		; Wait 1ms
	
	ret
	
;************************************************
;* DS1820 Write Byte	(4 Mhz)			*
;*						*
;* Input:	data				*
;* Requires:	temp (bit count)		*
;*		count1 (timer)			*
;************************************************

DS1820_Write:
	ldi	temp, 8			; 8 bits to write
	
DS1820_wloop:

	sbi	WIREDIR, WIRE		; [1] Bus low ( 1us to 15us )

	ldi	count2,13		; OH
wait_5u:
	nop				; 13 * (1+1+2)* 0.1 s = 0.4 s 
	dec	count2
	brne	wait_5u
	
	ror	data			; [1]
	brcc	DS1820_w0		; [1] Write 0

DS1820_w1:
	cbi	WIREDIR, WIRE		; Data = high

DS1820_w0:
					; Data alread low
	ldi	count1, 13 		; Just wait >60us
	rcall	delayX5us				
					
	cbi	WIREDIR, WIRE		; Bus high
	ldi	count1, 1		; Recovery time >1us
	rcall	delayX5us		; 5us delay!
	dec	temp				
	brne	DS1820_wloop
	
	ret						

;************************************************
;* DS1820 Read Byte	(4 Mhz)			*
;*						*
;* Output:	data				*
;* Requires:	temp (bit count)		*
;* 		count1 (timer)			*
;************************************************

DS1820_read:
	ldi	temp, 8			; 8 bits to write
	clr	data
DS1820_rloop:

	sbi	WIREDIR, WIRE		; [1] Bus low ( 1us to 15us )
	ldi	count1, 1		; Recovery time >1us
	rcall	delayX5us		; 5us delay!
	cbi	WIREDIR, WIRE		; Bus high
	ldi	count1, 1		; Recovery time >1us
	rcall	delayX5us		; 5us delay!
	; Get Data Now
	lsr	data
	sbic	WIREIN, WIRE		; check bit
	sbr	data, 0x80
					; Data alread inactive
	ldi	count1, 13		; Just wait >60us
	rcall	delayX5us				
									
	dec	temp
	brne	DS1820_rloop
	ret	
	
;************************************************
;* DS1820 Do Temp Conversion 	 		*
;*						*
;* Requires: DS1820_Write 			*
;************************************************

DS1820_conv:	
	ldi	data, 0xcc		; Skip ROM check
	rcall	DS1820_Write
	
	ldi	data, 0x44		; Start Temp Conversion
	rcall	DS1820_Write
	
wait_con:
	sbis	WIREIN, WIRE		; Conversion Done!
	rjmp	wait_con
	ret

;************************************************
;* DS1820 Read Scratch Pad		 	*
;* Requires DS1820 ROM ID subroutine		*
;* 						*
;************************************************

DS1820_Scr:

	ldi	data, 0xcc		; Skip ROM check
	rcall	DS1820_Write
	
	ldi	data, 0xbe		; read Scratch Pad
	rcall	DS1820_Write
	
  	; Byte 0
	rcall	DS1820_Read		; Read a byte
	mov	templsb, data		; store data
	;rcall	onechar
	
	; Byte 1
	rcall	DS1820_Read		; Read a byte
	mov	tempmsb, data		; store data
	;rcall	onechar
	
	; Byte 2
	rcall	DS1820_Read		; Read a byte
	mov	userb1, data		; store data
	;rcall	onechar
	
	; Byte 3
	rcall	DS1820_Read		; Read a byte
	mov	userb2, data		; store data
	;rcall	onechar
	
	; Byte 4
	rcall	DS1820_Read		; Read a byte
	mov	crap1, data		; store data
	;rcall	onechar
	
	; Byte 5
	rcall	DS1820_Read		; Read a byte
	mov	crap2, data		; store data
	;rcall	onechar
	
	; Byte 6
	rcall	DS1820_Read		; Read a byte
	mov	count_remain, data	; store data
	;rcall	onechar
	
	; Byte 7
	rcall	DS1820_Read		; Read a byte
	mov	count_per_c, data	; store data
	;rcall	onechar
																	
	; Byte CRC
	rcall	DS1820_Read		; Read a byte
	mov	scratch_crc, data	; store data
	;rcall	onechar
	
	ret
	
;************************************************
;* Dump Temp from SRAM to RS232 (real value)	*
;* Resolution 0.5 deg C	 negative works		*
;* 						*
;* Input:	templsb (SRAM)			*
;* Output:	RS232				*
;* Requires:	data				*
;*						*
;************************************************

dump_temp:

	; Check for positive or negative temp!

	mov	hex, templsb		; Get LSB
	ldi	data, ' '
	
	tst	hex
	brpl	postemp			; Temp is Positive!

	; Display a negative number!!

	ldi	data, '-'
	neg     hex

postemp:
	rcall	onechar
	
	lsr     hex			; remove 0.5 sens.
	rcall	hexdec			; Convert to decimal
	
	; display now
	
	;mov	data, templsb
	mov 	data, hund
	rcall 	onechar
	mov 	data, tens
	
	;mov	data, tempmsb
	rcall 	onechar
	mov 	data, unit
	rcall 	onechar
	
	; decimal place
	ldi	data, '.'
	rcall	onechar
	mov	hex, templsb
	ldi	data, '0'
	sbrc	hex, 0	
	ldi	data, '5'
	rcall	onechar
	ldi	data, 'C'
	rcall	onechar
	ret

